import re
from Crypto.Hash import SHA256
from Crypto.Cipher import AES
from base64 import b64decode
import binascii

# Ported from unobfuscated wrapKA function:
# https://gist.github.com/nhanb/74542c36d3dcc5dde4e90b34437fb523

# AFAIK this is hardcoded in http://kissmanga.com/Scripts/lo.js and is never
# redefined elsewhere so it's ok to hardcode it here too.
_iv = binascii.a2b_hex('a5e8e2e9c2721be0a84ad660c472c1f3')


def _generate_sha(plaintext):
    sha = SHA256.new()
    sha.update(plaintext.encode('utf-8'))
    return sha.digest()


# This is also found in lo.js
_default_key = _generate_sha('mshsdf832nsdbash20asdm')


def decode_url(input, key):
    encoded = b64decode(input)
    dec = AES.new(key=key, mode=AES.MODE_CBC, IV=_iv)
    result = dec.decrypt(encoded)
    unpadded = result[:-result[-1]]
    return unpadded.decode()


def _crypto_tag(tag):
    return tag.name == 'script' and tag.text and 'chko' in tag.text


def get_key(soup):
    '''
    On each chapter page, a <script> tag is inserted that overrides the
    encryption key, so we'll need to find that. Only fall back to default key
    if such a tag is not found.
    '''

    crypto_tag = soup.find(_crypto_tag)
    if crypto_tag is None:
        return _default_key

    pat = re.compile('\["(.+)"\]')
    keys = pat.findall(crypto_tag.text)
    if len(keys) > 0:
        unhashed_key = keys[-1].encode().decode('unicode_escape')
        return _generate_sha(unhashed_key)
    else:
        return _default_key



from .sites import get_site, available_sites
from .exceptions import UnsupportedSiteError


def parse_chapter(url):
    site = get_site(url)
    if site is None:
        raise UnsupportedSiteError()
    resp = site.get_chapter_seed_page(url)
    return site.chapter_info(resp.text)


def parse_series(url):
    site = get_site(url)
    if site is None:
        raise UnsupportedSiteError()
    resp = site.get_manga_seed_page(url)
    return site.series_info(resp.text)


def search_series(name):
    for site in available_sites.values():
        results = site.search_series(name)
        for series in results:
            yield series


def search_series_from_sites(name, site_names):
    for sname in site_names:
        results = available_sites[sname].search_series(name)
        for series in results:
            yield series


def search_series_from_site(name, site_name):
    site = available_sites[site_name]
    return [series for series in site.search_series(name)]



from urllib.parse import urlparse
from .kissmanga import Kissmanga
from .mangapanda import Mangapanda

_sites = [
    Kissmanga(),
    Mangapanda(),
]

available_sites = {site.name: site for site in _sites}


# Factory function, return instance of suitable "site" class from url
def get_site(url):
    netloc = urlparse(url).netloc
    for site in available_sites.values():
        if netloc in site.netlocs:
            return site
    return None



import unittest
from fundoshi.sites.kissmanga import Kissmanga
from . import _test_series, _test_chapter, _test_search_by_author, \
    _test_search_series
site = Kissmanga()


class TestKissmangaSearchSeries(unittest.TestCase):

    def test_beelzebub_search(self):
        expected = [
            {
                'name': 'Beelzebub',
                'url': 'http://kissmanga.com/Manga/Beelzebub',
                'site': 'kissmanga',
            },
            {
                'name': 'Beelzebub Bangai Hen',
                'url': 'http://kissmanga.com/Manga/Beelzebub-Bangai-Hen',
                'site': 'kissmanga',
            },
        ]
        _test_search_series(self, site, 'beelzebub', expected)

    def test_no_result_search(self):
        results = site.search_series('mot con vit xoe ra hai cai canh')
        self.assertListEqual(results, [])


class TestKissmangaSeries(unittest.TestCase):

    def test_completed_series(self):
        url = 'http://kissmanga.com/Manga/Beelzebub'
        expected = {
            'name': 'Beelzebub',
            'tags': ['action', 'comedy', 'fantasy', 'manga', 'school life',
                     'shounen', 'supernatural'],
            'thumb_url': 'http://kissmanga.com/Uploads/Etc/8-17-2011/75210229ef3c651a3.jpg',
            'status': 'completed',
            'descriptions': [
                'The story follows the "strongest juvenile delinquent" as he watches over '
                "the devil king's son (and future devil king) with the fate of the world "
                'hanging in the balance.'
            ],
            'authors': ['Tamura Ryuuhei'],
            'chapters': {
                'last': {
                    'url': 'http://kissmanga.com/Manga/Beelzebub/Babu-240?id=285551',
                    'name': 'Beelzebub Babu 240'
                },
                'first': {
                    'url': 'http://kissmanga.com/Manga/Beelzebub/Babu-000?id=285306',
                    'name': 'Beelzebub Babu 000',
                },
            },
        }
        _test_series(self, site, url, expected)


class TestKissmangaChapter(unittest.TestCase):

    def test_middle_chapter(self):
        url = 'http://kissmanga.com/Manga/Naruto/Naruto-635?id=290868'
        expected = {
            'name': 'Naruto 635',
            'prev_chapter_url': 'http://kissmanga.com/Manga/Naruto/Naruto-634?id=290867',
            'next_chapter_url': 'http://kissmanga.com/Manga/Naruto/Naruto-636?id=290869',
            'series_url': 'http://kissmanga.com/Manga/Naruto',
            'pages': [
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f01.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f02.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f03.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f04.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f05.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f06.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f07.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f08.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f09.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f10.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f11.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f12.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f13.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f14.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f15.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f16.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f17.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f18.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f19.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f20.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f21.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f22.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f23.jpg&imgmax=30000',
                'https://images1-focus-opensocial.googleusercontent.com/gadgets/proxy?container=focus&gadget=a&no_expand=1&resize_h=0&rewriteMime=image%2F*&url=http%3a%2f%2fcdn.eatmanga.com%2fmangas%2fManga-Scan%2fNaruto%2fNaruto-635%2f24.jpg&imgmax=30000',

            ],
        }
        _test_chapter(self, site, url, expected)

    def test_first_chapter(self):
        """First chapter has no prev_chapter_url"""

        url = 'http://kissmanga.com/Manga/5-Centimeters-per-Second/Vol-1-Ch-1-Read-Online?id=62872'
        expected = {
            'prev_chapter_url': None,
        }
        _test_chapter(self, site, url, expected)

    def test_last_chapter(self):
        """Last chapter has no next_chapter_url"""

        url = 'http://kissmanga.com/Manga/Shigatsu-wa-Kimi-no-Uso/Vol-011-Ch-044---End-?id=223537'
        expected = {
            'next_chapter_url': None,
        }
        _test_chapter(self, site, url, expected)


class TestKissmangaSearchByAuthor(unittest.TestCase):

    def test_normal_author(self):
        _test_search_by_author(self, site, 'ARAKAWA Naoshi', [
            {
                'url': 'http://kissmanga.com/Manga/Sayonara-Football',
                'site': 'kissmanga',
                'name': 'Sayonara Football',
            },
            {
                'url': 'http://kissmanga.com/Manga/Shigatsu-wa-Kimi-no-Uso',
                'site': 'kissmanga',
                'name': 'Shigatsu wa Kimi no Uso',
            },
            {
                'url': 'http://kissmanga.com/Manga/Shigatsu-wa-Kimi-no-Uso-Coda',
                'site': 'kissmanga',
                'name': 'Shigatsu wa Kimi no Uso - Coda',
            },
        ])



from bs4 import BeautifulSoup
from .utils import cfscraper
from fundoshi.classes import objdict

_get = cfscraper.get
_post = cfscraper.post


class Mangapanda(object):

    netlocs = ['www.mangapanda.com']
    name = 'mangapanda'

    def __init__(self):
        # mangapanda uses a lot of relative paths
        self.base_url = 'http://www.mangapanda.com'

    # Return a list of objdicts that store name, url, and site:
    # [{'name': 'Naruto', 'url': 'http://...', 'site': mangapanda}, ...]
    def search_series(self, keyword):
        url = 'http://www.mangapanda.com/search/'
        params = {
            'w': keyword,
        }
        resp = _post(url, params=params)

        soup = BeautifulSoup(resp.content, 'html.parser')
        manga_names = soup.find_all('div', {'class': 'manga_name'})
        atags = [name.find('h3').a for name in manga_names]
        return [objdict({'name': a.text.strip(),
                         'url': self.base_url + a['href'],
                         'site': self.name}) for a in atags]

    # All kinds of data
    # - name "Naruto"
    # - chapters [{name, url}, {}, ...] - latest first
    # - thumb_url "url"
    # - tags [tag1, tag2, ...]
    # - status "ongoing"/"completed"
    # - descriptions ["paragraph1", "paragraph2", ...]
    # - authors ["Kishimoto Masashi", ...]
    def series_info(self, html):
        soup = BeautifulSoup(html, 'html.parser')
        chapters = self._chapters(soup)
        thumb_url = self._thumbnail_url(soup)
        tags = self._tags(soup)
        name = self._name(soup)
        status = self._status(soup)
        descriptions = self._descriptions(soup)
        authors = self._authors(soup)
        return objdict({
            'site': self.name,
            'chapters': chapters,
            'thumb_url': thumb_url,
            'tags': tags,
            'name': name,
            'status': status,
            'descriptions': descriptions,
            'authors': authors,
        })

    def _chapters(self, soup):
        table = soup.find('table', {'id': 'listing'})
        return [objdict({'url': self.base_url + a['href'],
                         'name': a.string.strip()})
                for a in reversed(table.find_all('a'))]

    def _thumbnail_url(self, soup):
        img_div = soup.find('div', {'id': 'mangaimg'})
        return img_div.find('img')['src']

    def _tags(self, soup):
        tag_list = soup.find(
            'td',
            {'class': 'propertytitle'},
            text='Genre:'
        ).findNext('td')
        tags = tag_list.find_all('a')
        return [text.string.strip().lower() for text in tags]

    def _name(self, soup):
        # <link rel='alternate' title='Naruto manga' ...
        # => must remove the ' manga' part
        title = soup.find('h2', {'class': 'aname'})
        return title.text.strip()

    def _status(self, soup):
        status_td = soup.find(
            'td',
            {'class': 'propertytitle'},
            text='Status:'
        ).findNext('td')
        return status_td.text.strip().lower()

    def _descriptions(self, soup):
        desc_div = soup.find('div', {'id': 'readmangasum'})
        p_tag = desc_div.findNext('p')
        desc = p_tag.text.replace('\r\n', '')
        return [desc]

    def _authors(self, soup):
        # mangapanda lists artists in some cases
        # lets strip these as well as the tags used
        authors = soup.find(
            'td',
            {'class': 'propertytitle'},
            text='Author:'
        ).findNext('td')
        return [authors.text.strip()]

    # Chapter data
    # - name "Naruto Ch.101"
    # - pages [url1, url2, ...] - in ascending order
    # - prev_chapter_url
    # - next_chapter_url
    # - series_url
    def chapter_info(self, html):
        pages = self._chapter_pages(html)
        soup = BeautifulSoup(html, 'html.parser')
        name = self._chapter_name(soup)
        series_url = self._chapter_series_url(soup)
        prev, next = self._chapter_prev_next(soup)
        return objdict({
            'name': name,
            'pages': pages,
            'series_url': series_url,
            'next_chapter_url': next,
            'prev_chapter_url': prev,
        })

    def _chapter_prev_next(self, soup):
        chapter_nav = soup.find('div', {'id': 'mangainfo_bas'}).table
        # There is always a "next page" on Mangapanda even if no release exists
        # so we check every page for the not released notification
        # before setting the next page link
        next_page = chapter_nav.find('span', text='Next Chapter:')
        next_page = self.base_url + next_page.findNext('a')['href']
        look_ahead = _get(next_page)
        look_ahead_soup = BeautifulSoup(look_ahead.text, 'html.parser')
        info = look_ahead_soup.find('div', {'id': 'recom_info'})
        if info and 'not released' in info.text:
            next_page = None
        prev_page = chapter_nav.find('span', text='Previous Chapter:')
        if prev_page:
            prev_page = self.base_url + prev_page.findNext('a')['href']
        return prev_page, next_page

    def _chapter_name(self, soup):
        chapter_info = soup.find('div', {'id': 'mangainfo'})
        chapter_name = chapter_info.h1
        return chapter_name.text.strip()

    def _chapter_pages(self, html):
        soup = BeautifulSoup(html, 'html.parser')
        # get last page number
        select_page = soup.find('div', {'id': 'selectpage'})
        last_page = int(select_page.text.strip().rsplit('of ', 1)[-1])

        # get base_url of chapter
        manga_info = soup.find('div', {'id': 'mangainfo_son'})
        chapter_base = manga_info.find('a')['href']

        # generate page urls
        page_urls = []
        for page_num in range(1, last_page + 1):
            page_urls.append('{0}{1}/{2}'.format(self.base_url,
                                                 chapter_base,
                                                 page_num))

        return (self._parse_page(url) for url in page_urls)

    def _parse_page(self, page_url):
        response = _get(page_url)
        soup = BeautifulSoup(response.text, 'html.parser')
        # get img url
        img_holder = soup.find('div', {'id': 'imgholder'})
        return img_holder.img['src']

    def _chapter_series_url(self, soup):
        manga_info = soup.find('div', {'id': 'mangainfo_son'})
        a_tag = manga_info.find_all('a')[-1]
        return self.base_url + a_tag['href']

    def search_by_author(self, author):
        # This site simply doesn't support searching by author
        return []

    def get_manga_seed_page(self, url):
        return _get(url)

    def get_chapter_seed_page(self, url):
        # http://www.mangapanda.com/Manga-Name/X
        base = url[:url.rfind('/') + 1]  # http://.../Manga-Name/
        chapter = url.rsplit('/', 1)[-1]  # /X
        return _get(base + chapter, cookies={'vns_readType1': '1'})



import unittest
from fundoshi.sites.mangapanda import Mangapanda
from . import (
    _test_series,
    _test_chapter,
    _test_search_series
)
site = Mangapanda()


class TestMangapandaSearchSeries(unittest.TestCase):

    def test_beelzebub_search(self):
        expected = [
            {
                'name': 'Beelzebub',
                'url': 'http://www.mangapanda.com/beelzebub',
                'site': 'mangapanda',
            },
            {
                'name': 'Beelzebub Side Story',
                'url': 'http://www.mangapanda.com/beelzebub-side-story',
                'site': 'mangapanda',
            },
        ]
        _test_search_series(self, site, 'beelzebub', expected)

    def test_no_result_search(self):
        results = site.search_series('mot con vit xoe ra hai cai canh')
        self.assertListEqual(results, [])


class TestMangapandaSeries(unittest.TestCase):

    def test_completed_series(self):
        url = 'http://www.mangapanda.com/beelzebub'
        expected = {
            'name': 'Beelzebub',
            'tags': ['action', 'comedy', 'demons', 'fantasy', 'school life',
                     'shounen', 'supernatural'],
            'thumb_url': 'http://s4.mangapanda.com/cover/beelzebub/beelzebub-l0.jpg',
            'status': 'ongoing',
            'descriptions': [
                'The story follows the strongest juvenile delinquent, Oga Tatsumi, a first year in Ishiyama High the school for delinquents. One day while sleeping next to a river he sees a man floating down it, he pulls him to shore and the man splits in half revealing a baby boy. This boy is the son of the demon king and he has been chosen as the one to raise it with the baby s demon maid Hilda. The story follows his life with the child and at the delinquent school.[Source: Wiki]',
            ],
            'authors': ['Tamura, Ryuuhei (Story & Art)'],
            'chapters': {
                'last': {
                    'url': 'http://www.mangapanda.com/beelzebub/240',
                    'name': 'Beelzebub 240'
                },
                'first': {
                    'url': 'http://www.mangapanda.com/beelzebub/1',
                    'name': 'Beelzebub 1',
                },
            },
        }
        _test_series(self, site, url, expected)


class TestMangapandaChapter(unittest.TestCase):

    def test_middle_chapter(self):
        url = 'http://www.mangapanda.com/naruto/699'
        expected = {
            'name': 'Naruto 699',
            'prev_chapter_url': 'http://www.mangapanda.com/naruto/698',
            'next_chapter_url': 'http://www.mangapanda.com/naruto/700',
            'series_url': 'http://www.mangapanda.com/naruto',
            'pages': [
                'http://i9.mangapanda.com/naruto/699/naruto-5283045.jpg',
                'http://i1.mangapanda.com/naruto/699/naruto-5283047.jpg',
                'http://i1.mangapanda.com/naruto/699/naruto-5283049.jpg',
                'http://i5.mangapanda.com/naruto/699/naruto-5283051.jpg',
                'http://i9.mangapanda.com/naruto/699/naruto-5283053.jpg',
                'http://i9.mangapanda.com/naruto/699/naruto-5283055.jpg',
                'http://i7.mangapanda.com/naruto/699/naruto-5283057.jpg',
                'http://i5.mangapanda.com/naruto/699/naruto-5283059.jpg',
                'http://i8.mangapanda.com/naruto/699/naruto-5283061.jpg',
                'http://i2.mangapanda.com/naruto/699/naruto-5283063.jpg',
                'http://i10.mangapanda.com/naruto/699/naruto-5283065.jpg',
                'http://i6.mangapanda.com/naruto/699/naruto-5283067.jpg',
                'http://i8.mangapanda.com/naruto/699/naruto-5283069.jpg',
                'http://i10.mangapanda.com/naruto/699/naruto-5283071.jpg',
                'http://i2.mangapanda.com/naruto/699/naruto-5283073.jpg',
                'http://i2.mangapanda.com/naruto/699/naruto-5283075.jpg',
                'http://i10.mangapanda.com/naruto/699/naruto-5283077.jpg',
                'http://i2.mangapanda.com/naruto/699/naruto-5283079.jpg',
                'http://i8.mangapanda.com/naruto/699/naruto-5283081.jpg',
                'http://i2.mangapanda.com/naruto/699/naruto-5283083.jpg',
                'http://i4.mangapanda.com/naruto/699/naruto-5283085.jpg',
            ],
        }
        _test_chapter(self, site, url, expected)

    def test_first_chapter(self):
        """First chapter has no prev_chapter_url"""

        url = 'http://www.mangapanda.com/naruto/1'
        expected = {
            'prev_chapter_url': None,
        }
        _test_chapter(self, site, url, expected)

    def test_last_chapter(self):
        """Last chapter has no next_chapter_url"""

        url = 'http://www.mangapanda.com/naruto/700'
        expected = {
            'next_chapter_url': None,
        }
        _test_chapter(self, site, url, expected)



import re
from bs4 import BeautifulSoup
import bs4
from fundoshi.classes import objdict
from ..utils import cfscraper
from .decoder import decode_url, get_key

_get = cfscraper.get
_post = cfscraper.post


class Kissmanga(object):

    netlocs = ['kissmanga.com']
    name = 'kissmanga'

    # Return a list of objdicts that store name, url, and site:
    # [{'name': 'Naruto', 'url': 'http://...', 'site': kissmanga}, ...]
    def search_series(self, keyword):
        url = 'http://kissmanga.com/Search/SearchSuggest'
        params = {
            'type': 'Manga',
            'keyword': keyword,
        }
        resp = _post(url, params=params)

        # Kissmanga returns manga series and links in xml format
        soup = BeautifulSoup(resp.content, 'html.parser')
        atags = soup.find_all('a')
        return [objdict({'name': a.text.strip(),
                         'url': a['href'],
                         'site': self.name}) for a in atags]

    # All kinds of data
    # - name "Naruto"
    # - chapters [{name, url}, {}, ...] - latest first
    # - thumb_url "url"
    # - tags [tag1, tag2, ...]
    # - status "ongoing"/"completed"
    # - descriptions ["paragraph1", "paragraph2", ...]
    # - authors ["Kishimoto Masashi", ...]
    def series_info(self, html):
        soup = BeautifulSoup(html, 'html.parser')
        chapters = self._chapters(soup)
        thumb_url = self._thumbnail_url(soup)
        tags = self._tags(soup)
        name = self._name(soup)
        status = self._status(soup)
        descriptions = self._descriptions(soup)
        authors = self._authors(soup)
        return objdict({
            'site': self.name,
            'chapters': chapters,
            'thumb_url': thumb_url,
            'tags': tags,
            'name': name,
            'status': status,
            'descriptions': descriptions,
            'authors': authors,
        })

    def _chapters(self, soup):
        table = soup.find('table', class_='listing')
        return [objdict({'url': 'http://kissmanga.com' + a['href'],
                         'name': a.string.strip()})
                for a in table.find_all('a')]

    def _thumbnail_url(self, soup):
        return soup.find('link', {'rel': 'image_src'})['href']

    def _tags(self, soup):
        tags = soup.find('span', text='Genres:').find_next_siblings('a')
        return [text.string.strip().lower() for text in tags]

    def _name(self, soup):
        # <link rel='alternate' title='Naruto manga' ...
        # => must remove the ' manga' part
        return soup.find('link', {'rel': 'alternate'})['title'][:-6]

    def _status(self, soup):
        status_span = soup.find('span', {'class': 'info'}, text='Status:')
        return status_span.next_sibling.strip().lower()

    def _descriptions(self, soup):
        desc_span = soup.find('span', {'class': 'info'}, text='Summary:')
        p_tags = desc_span.next_siblings
        desc = [s.text for s in p_tags if type(s) == bs4.element.Tag]
        return desc

    def _authors(self, soup):
        authors = soup.find('span', text='Author:').find_next_siblings('a')
        return [text.string.strip() for text in authors]

    # Chapter data
    # - name "Naruto Ch.101"
    # - pages [url1, url2, ...] - in ascending order
    # - prev_chapter_url
    # - next_chapter_url
    # - series_url
    def chapter_info(self, html):
        pages = self._chapter_pages(html)
        soup = BeautifulSoup(html, 'html.parser')
        name = self._chapter_name(soup)
        series_url = self._chapter_series_url(soup)
        prev, next = self._chapter_prev_next(soup)
        return objdict({
            'name': name,
            'pages': pages,
            'series_url': series_url,
            'next_chapter_url': next,
            'prev_chapter_url': prev,
        })

    def _chapter_prev_next(self, soup):
        next = soup.find('img', class_='btnNext')
        if next is not None:
            next = next.parent['href']
        prev = soup.find('img', class_='btnPrevious')
        if prev is not None:
            prev = prev.parent['href']
        return prev, next

    def _chapter_name(self, soup):
        select = soup.find('select', class_='selectChapter')
        return select.find('option', selected=True).text.strip()

    def _chapter_pages(self, html):
        soup = BeautifulSoup(html, 'html.parser')
        key = get_key(soup)

        pat = re.compile('lstImages\.push\(wrapKA\("(.+?)"\)\);')
        encrypted_urls = pat.findall(html)
        return (decode_url(url, key) for url in encrypted_urls)

    def _chapter_series_url(self, soup):
        a_tag = soup.find('div', id='navsubbar').find('p').find('a')
        return a_tag['href']

    def get_manga_seed_page(self, url):
        return _get(url)

    def get_chapter_seed_page(self, url):
        # http://kissmanga.com/Manga/Manga-Name/Chapter-Name?id=XXXXXXX
        # The "Chapter-Name" part can be any non-empty string and the response
        # will be the same. Problem is that it sometimes contains illegal chars
        # that make pytaku's request fail. Let's replace it with something
        # safe.
        base = url[:url.rfind('/') + 1]  # http://.../Manga-Name/
        id = url[url.rfind('?id='):]  # ?id=XXXXXX
        return _get(base + '_' + id, cookies={'vns_readType1': '1'})

    def search_by_author(self, author):
        url = 'http://kissmanga.com/AuthorArtist/' + author.replace(' ', '-')
        resp = _get(url)

        soup = BeautifulSoup(resp.content, 'html.parser')
        table = soup.find('table', class_='listing')

        if table is None:  # no author of this name
            return []

        return [objdict({
            'name': a.text.strip(),
            'url': 'http://kissmanga.com' + a['href'],
            'site': self.name,
        }) for a in table.find_all('a') if len(a['href'].split('/')) == 3]



class objdict(dict):
    """
    Dictionary that lets you access its values like object attributes.
    """

    def __getattr__(self, name):
        if name in self:
            return self[name]
        else:
            raise AttributeError("No such attribute: " + name)

    def __setattr__(self, name, value):
        self[name] = value

    def __delattr__(self, name):
        if name in self:
            del self[name]
        else:
            raise AttributeError("No such attribute: " + name)



import os
from setuptools import setup, find_packages


def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()


setup(
    name='fundoshi',
    version='0.0.8',
    description='Get manga series & chapter data from various reader sites.',
    long_description=read('README.rst'),
    url='http://github.com/nhanb/fundoshi',
    license='MIT',
    author='Bùi Thành Nhân',
    author_email='nhan@nerdyweekly.com',
    packages=find_packages(exclude=['tests*']),
    install_requires=['beautifulsoup4', 'requests', 'cfscrape', 'pycrypto'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
